#ifndef _SWAP_H__
#define _SWAP_H__

#include "TNT\TNT.h"
#include "datecl.h"
#include <string>
#include <vector>
#include <map>
#include <iostream>
#define NUM_DATES        100
#define THIRTY            30
#define THREE_SIXTY      360
#define THREE_SIXTY_FIVE 365
#define NOTIONAL         1000000
using namespace std;
static std::vector<Date> payDates_;

static double interpolate(double rate1, double rate2, double t1, double t2, double x) {
   double dy = rate2 - rate1;
   double dt = t2 - t1;
   double slope = dy/dt;

   return rate1 + slope*x; 
}

class FloatingLeg
{
 public:
    FloatingLeg(std::map<double,double> floatLegRate, double floatLegBasis, int payFrequency) 
 		: floatLegRate_(floatLegRate),floatLegBasis_(floatLegBasis),payFrequency_(payFrequency) {}
    FloatingLeg() {}
    virtual ~FloatingLeg() {}
    inline double calcDuration() {  
      double duration = 0.0;
      duration = (double) (payDates_[0] - valuationDate_ + 1)/THREE_SIXTY_FIVE; //(maturityDate_ - valuationDate_ + 1);      
	//(paysum/val;
      return duration;
    }
    inline void setEffectiveDate(Date date) { startDate_ = date; }
    inline void setValuationDate(Date date) { valuationDate_ = date; }
    inline void setNotional(double notional) { notional_ = notional; }
    inline Date getEffectiveDate() { return startDate_; }
    inline void setFrequency(int frequency) { payFrequency_ = frequency; }
    inline void setMaturityDate(Date mat) { maturityDate_ = mat; }
    inline std::vector<double> getPayFloat() { return floatRates; }
    inline void setFloatValue(double value) { value_ = value; }
    inline double getFloatValue() { 
	return value_; 
    }
    inline void setFloatRate(std::map<double,double> rate) {
	floatLegRate_ = rate;
    }
    inline double calcDV01() {

	double duration = calcDuration();
	double val = getFloatValue();
	double DV = 0.0;

	DV = -(duration*notional_)*((double)1/10000); 
	cout << "float DV01 = " << DV << endl;

	return -DV;
    }
    inline double calcModifiedDuration() {

	double val = calcDV01();
	double marketValue = getFloatValue();
	double MD = (val/(notional_ + marketValue))*10000;

	cout << "float modified duration = " << MD << endl;
		
	return MD;
     }
     inline void calcPayFloat() {
		
	std::vector<Date>::iterator iter;
	double val = 0;
	double diff = 0.0;
	int diff1 = 0.0;
	int d = 0.0;
	int d1 = 0.0;
	Date dateDiff;
	int cnt = 0;
	double floatVal = 0.0;
		
	for (iter = payDates_.begin(); iter != payDates_.end(); iter++)
	{
	     d = payDates_[cnt+1] - payDates_[0] + 1;
	     d1 = payDates_[cnt+1] - payDates_[cnt] + 1;
			
	     diff = payDates_[cnt+1] - payDates_[0]+1;
	     diff = (double) diff/THREE_SIXTY_FIVE;
	     floatVal = interpolate(floatLegRate_[floor(diff)],floatLegRate_[ceil(diff)],floor(diff),ceil(diff),diff);	
		
	     if (payFrequency_ == 1)
		val = notional_*(THIRTY/THREE_SIXTY)*floatVal;
	     else 
		val = notional_*((double)d1/THREE_SIXTY_FIVE)*floatVal;
			
	     floatRates.push_back(val);
			
	     cnt++;
	 }	
     }
   private:
        double floatLegBasis_;
	std::vector<double> floatRates;
	std::map<double,double> floatLegRate_;
	std::map<double,double> payfloatLeg_;
	Date startDate_;
	Date maturityDate_;
	Date valuationDate_;
	double value_;
	double spread_;
	double notional_;
	double duration_;
	double accrual_;
	int payFrequency_;
};

class FixedLeg
{
   public:
	FixedLeg(double payfixedLeg, double fixedLegRate, double fixedLegBasis, int payFrequency) 
		: payfixedLeg_(payfixedLeg), fixedLegRate_(fixedLegRate), fixedLegBasis_(fixedLegBasis),
		payFrequency_(payFrequency) { }
	FixedLeg() {}
	virtual ~FixedLeg() {}
	inline double calcDV01() {

	  double duration = calcDuration();
	  double val = getFixedValue(); 
	  double DV = 0.0;

	  DV = (duration*notional_)*((double)1/10000); 
	  cout << "fixed DV01 = " << DV << endl;

	  return DV;
	}
	inline void setEffectiveDate(Date date) { effectiveDate_ = date; }
	inline double calcDuration() { 

	  double sum = 0;
	  double val = getFixedValue();
	  double duration = 0.0;
	  double dis = 0.0;
		
	  for (int i = 0; i < payfixedLeg_.size(); i++)
	     sum = sum + payfixedLeg_[i]*((double)(payDates_[i+1] - valuationDate_ + 1)/
		(maturityDate_ - valuationDate_ + 1));

	     sum = sum + notional_;
	     duration = sum/val;
	     cout << "fixed duration = " << duration << endl;

	     return duration;
	}
	inline double calcModifiedDuration() {

	   double val = calcDV01();
	   double marketValue = getFixedValue();
	   double MD = (val/(notional_ + marketValue))*10000;

	    //cout << "fixed modified duration = " << MD << endl;
		
	   return MD;
	}
	inline void setFixedValue(double val) { value_ = val; }
	inline void setValuationDate(Date date) { valuationDate_ = date; }
	inline double getFixedValue() {
	   return value_; 
	}
	inline void setNotional(double notional) { notional_ = notional; }
	inline void setFrequency(int frequency) { payFrequency_ = frequency; }
	inline void setFixedRate(double rate) { fixedLegRate_ = rate; }
	inline void setMaturityDate(Date mat) { maturityDate_ = mat; }
	std::vector<double> getPayFixed() { return payfixedLeg_; }
	inline void calcPayFixed() 
	{	
	   std::vector<Date>::iterator iter;
	   double val = 0;
	   int diff = 0;
	   Date dateDiff;
	   int cnt = 0;
		
	   for (iter = payDates_.begin(); iter != payDates_.end(); iter++)
	   {
	      if (payFrequency_ == 1)
	      {
		   if (cnt <= payDates_.size())
		   {
			if (cnt + payFrequency_ < payDates_.size())
			   diff = payDates_[cnt+payFrequency_] - payDates_[cnt]+1;
			else
			   diff = 0;
		
			if ((cnt != 0) && (cnt % payFrequency_ == 0))
			    val = notional_*(THIRTY/THREE_SIXTY)*fixedLegRate_;
			else
			    val = 0;
	           }
	           else
		      val = notional_*(THIRTY/THREE_SIXTY)*fixedLegRate_;
	      } 
	      else 
	      {
		if (cnt <= payDates_.size()) 
		{
		   if (cnt + payFrequency_ <= payDates_.size())
		   {
		      // subtract five because there are 5 less days in a 360 day year
			diff = (payDates_[cnt+payFrequency_] - payDates_[cnt] + 1) - 5;
			//cout << "diff = " << diff << endl;
	 	   }
		   else
		      diff = 0;
		
		   if ((cnt > 0) && ((cnt-1) % payFrequency_ == 0)) 
		      val = notional_*((double)diff/THREE_SIXTY)*fixedLegRate_;
		   else
		      val = 0;
		}
		else
		{
		   //diff = (maturityDate_ - payDates_[cnt] + 1) - 5;
		   //val = notional_*((double)diff/THREE_SIXTY)*fixedLegRate_;
		   val = 0;
		}
		//cout << "fixed val = " << val << "cnt = " << cnt << endl;
	       }
	       cnt++;
	       payfixedLeg_.push_back(val);
	  } // for
	}
   private:
	std::vector<double> payfixedLeg_;
	double fixedLegRate_;
	double fixedLegBasis_;
	double duration_;
	double value_;
	double accrual_;
	double notional_;
	double basis_;
        int payFrequency_;
	Date effectiveDate_;
	Date maturityDate_;
	Date valuationDate_;
};

class Swap
{
    public:
	Swap() : notional_(NOTIONAL), maturity_("12/31/2010"), swapType(0) {}
	Swap(double notional, Date maturity, Date effectiveDate, Date settlementDate, Date valuation, 
	std::map<double,double> liborRate, std::map<double,double> disc, double fixedRate, int type) 
	   : notional_(notional), maturity_(maturity), effectiveDate_(effectiveDate), settlementDate_(settlementDate), 		valuationDate_(valuation), floatRates_(liborRate), fixedRate_(fixedRate), swapType(type)
	{
	   getNotional();
	   calcPayDates(effectiveDate_,maturity_,valuationDate_);
	   fixedLeg.setNotional(notional_);
	   fixedLeg.setValuationDate(valuationDate_);
	   fixedLeg.setFrequency(2);
	   fixedLeg.setFixedRate(fixedRate);
	   fixedLeg.setMaturityDate(maturity);
	   fixedLeg.calcPayFixed();
	   fixedLeg.setEffectiveDate(effectiveDate_);
	   floatLeg.setMaturityDate(maturity);
	   floatLeg.setFrequency(4);
	   floatLeg.setNotional(notional_);
	   floatLeg.setFloatRate(liborRate);
	   floatLeg.setValuationDate(valuationDate_);
	   setDiscountRates(disc);
	   floatLeg.calcPayFloat();
	   netPayments();
	   //fixedLeg.calcModifiedDuration();
	   //floatLeg.calcModifiedDuration();
	   calcDV01();			}
	   virtual ~Swap() {}
	   inline double getNotional() {
		return notional_;
	   }
	   inline void setDiscountRates(std::map<double,double> rate) {
	      discRates_ = rate;
	   }
	   inline double calcDV01() {  
		
	      double val = 0;
			
	      if (swapType == 0)
		 val = fixedLeg.calcDV01() - floatLeg.calcDV01(); 
	      else
		val = floatLeg.calcDV01() - fixedLeg.calcDV01();

	      cout << "Swap DV01 = " << val << endl << endl;

	      return val;
	   }
	   inline void calcPayDates(Date tradeDate, Date endDate, Date valuation) 
	   {		
	      effectiveDate_ = tradeDate-1;
			
	      if (effectiveDate_ == Date::SATURDAY)
		 effectiveDate_ = effectiveDate_ + 2;
	      else if (effectiveDate_ == Date::SUNDAY)
		 effectiveDate_ = effectiveDate_ + 1;
			
	      Date currDate = effectiveDate_;
	      int cnt = 0;
			
	      while (currDate <= endDate)  
	      {
		   currDate.AddMonths(3);
		   while (currDate.day > effectiveDate_.day)
			currDate = currDate - 1;
						
		   if (currDate <= valuation)
		   {
			if (currDate.day_of_week == Date::SATURDAY)
			   currDate = currDate + 2;
			else if (currDate.day_of_week == Date::SUNDAY) 
			   currDate = currDate + 1;
			else if (currDate == currDate.ChristmasDay(currDate.year))
			   currDate = currDate + 1;
						
			fixedAccruedDate_ = currDate;
		    }
		    if ((currDate <= endDate) && (currDate >= valuation))
		    {
			if (currDate.day_of_week == Date::SATURDAY)
			   currDate = currDate + 2;
			else if (currDate.day_of_week == Date::SUNDAY) 
			   currDate = currDate + 1;
			else if (currDate == currDate.ChristmasDay(currDate.year))
			   currDate = currDate + 1;
			
			payDates_.push_back(currDate);

			//cout << "payDate = " << payDates_[cnt] << endl;
			cnt++;
		     }
		}
	    }
	    inline void netPayments() 
	    {
		double val = 0.0;
		double y = 0.0;
		std::vector<double> fixed = fixedLeg.getPayFixed();
		std::vector<double> fl = floatLeg.getPayFloat();
		double x = 0;
		double sum = 0.0;
		double sumfix = 0.0;
		double sumfloat = 0.0;

		if (swapType == 0)
		{
		   fixedAccrued_ =  notional_*fixedRate_*((valuationDate_ - fixedAccruedDate_))/THREE_SIXTY;
		   floatAccrued_ = -notional_*floatRates_[0]*((valuationDate_ - fixedAccruedDate_))/THREE_SIXTY_FIVE;         
		}
		else
		{
		   fixedAccrued_ = -notional_*fixedRate_*((valuationDate_ - fixedAccruedDate_))/THREE_SIXTY;
		   floatAccrued_ =  notional_*floatRates_[0]*((valuationDate_ - fixedAccruedDate_))/THREE_SIXTY_FIVE;         
		}

		int cnt = 0;
		for (int i = 0; i < payDates_.size(); i++)
		{
		   x = (double) (payDates_[i+1] - payDates_[0] + 1)/THREE_SIXTY_FIVE;
		   //if (x > 1)
		   y = interpolate(discRates_[floor(x)],discRates_[ceil(x)],floor(x),ceil(x),x);
		   //else 
		   //	y = interpolate(discRates_[floor(x)],discRates_[ceil(x)-0.5],floor(x),ceil(x)-0.5,x);
		   //cout << "y = " << y << endl;

		   if (swapType == 0)
		   {
		      val = (fixed[i] - fl[i])*y;
		      sumfix = sumfix + fixed[i]*y;
		      sumfloat = sumfloat - fl[i]*y;
		   }
		   else
		   {
		      val = (fl[i] - fixed[i])*y;
		      sumfix = sumfix - fixed[i]*y;
		      sumfloat = sumfloat + fl[i]*y;
		   }	
		   sum = sum + val;
		
		   //cout << "payDates = " << payDates_[i] << " " << "net PV = " << val << endl;
		}
		fixedLeg.setFixedValue(sumfix);
		floatLeg.setFloatValue(sumfloat);
			
		cout << "Fixed Accrued = " << fixedAccrued_ << endl;
		cout << "Float Accrued = " << floatAccrued_ << endl;
		cout << "Accrued = " << fixedAccrued_ + floatAccrued_ << endl;
		cout << "Principal = " << sum << endl;
		cout << "Market Value = " << sum + (fixedAccrued_ + floatAccrued_) << endl;
	     }
	private:
	  int swapType;
	  double notional_;
	  double fixedAccrued_;
	  double floatAccrued_;
	  double fixedRate_;
	  FloatingLeg floatLeg;
	  FixedLeg fixedLeg;
	  Date maturity_;
	  Date fixedAccruedDate_;
	  Date floatAccruedDate_;
	  Date effectiveDate_;
	  Date settlementDate_;
	  Date valuationDate_;
	  std::string index;
	  std::map<double,double> discRates_;
	  std::map<double,double> floatRates_;
	  double value;
};

#endif _SWAP_H__
